<?php

class LOGReaderData extends ArrayIterator {
 private $list;
 private $items;
 private $modify_key = false;

 public function __construct($value, $items) {
    if (array_key_exists(0, $value)) {
	$this->modify_key = true;
    }

    $this->items = &$items;
    $this->list = array();

    foreach ($this->items as &$info) {
	$this->list[$info['logid']] = true;
    }

    parent::__construct($value);
 }
 
 public function key() {
    if ($this->modify_key) {
	$res = parent::current();
	return $res['time']->format('U.u');
    }
    return  parent::key();
 }

 public function current() {
    $logs = parent::current();

    $res = array();
    foreach ($this->items as $key => &$info) {
	$id = $info['logid'];
	if (array_key_exists($id, $logs)) {
	    $val = $logs[$id];
	    if (!is_numeric($val)) $val = crc32($val);
	    $res[$key] = $val;
	} else if ($id == "*") {
	    $sum = 0;
	    foreach ($logs as $id => $val) {
		if (!isset($this->list[$id])) $sum += $val;
	    }
	    $res[$key] = $sum;
	} else {
	    $res[$key] = NULL;
	}
    }

    return $res;
 }
}

class LOGReader extends READER {
 var $cache;

 var $groups;
 var $filter;
 var $params;

 function __construct(&$props) {
    parent::__construct($props);
    
    $this->groups = $this->opts->Get('groups');
 }

 function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0) {
    $groups = array();
    foreach ($this->groups as $gid => &$group) {
	if (($grp)&&(strcmp($grp->gid, $gid))) continue;

	$groups[$gid] = array(
	    'gid' => $gid,
	    'name' => $group['name']
	);
    
	if ($flags&REQUEST::NEED_INFO) {
	    $limit = $this->opts->GetDateLimit(0, time());
	    $groups[$gid]['first'] = $limit[0]; 
	    $groups[$gid]['last'] = $limit[1]; 
/*
	    if ($flags&REQUEST::NEED_COUNT) {
		$groups[$gid]['records'] = 
	    }
*/

	    if ($flags&REQUEST::NEED_ITEMINFO) {
		if ($grp) $grtest = $grp;
		else {
		    $ginfo = array("db_group" => $gid);
		    $grtest = $this->CreateGroup($ginfo);
		}

		$groups[$gid]['items'] = $this->GetItemList($grtest);
	    }
	}
    }

    if (($grp)&&(!$groups)) {
	throw new ADEIException(translate("Invalid group (%s) is specified", $grp->gid));
    }

    return $grp?$groups[$grp->gid]:$groups;
 }
 
 
 function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0) {
    if ($flags&REQUEST::ONLY_AXISINFO) {
	if (!$this->req->GetGroupOptions($grp, "axis")) return array();
    }

    $grp = $this->CheckGroup($grp, $flags);
    if (!$mask) $mask = $this->CreateMask($grp, $info=NULL, $flags);


    $items = $this->groups[$grp->gid]['items'];

    $res = array();
    for ($i = 0, $pos = 0; isset($items[$i]); $i++) {
	if (!$mask->Check($i)) continue;

	$name = (($items[$i]=="*")?"others":$items[$i]);
	$res[$pos] = array(
	    "id" => $i,
	    "uid" => $grp->gid . "__" . $name,
	    "name" =>  $name,
	    "logid" => $items[$i]
	);

	$pos++;
    }

    if ($flags&REQUEST::NEED_AXISINFO) {
	$this->AddAxisInfo($grp, $res);
    }

    return $res;
 }

 function GetRawData(LOGGROUP $grp = NULL, $from = 0, $to = 0, DATAFilter $filter = NULL, &$filter_data = NULL) {
    $grp = $this->CheckGroup($grp);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);

    if ($filter) {
	$mask = $filter->GetItemMask();
	$resample = $filter->GetSamplingRate();
	$limit = $filter->GetVectorsLimit();
	if ($limit) $ivl->SetItemLimit($limit);

	if (isset($filter_data)) {
	    if ($mask) $filter_data['masked'] = true;
//	    if ($resample) $filter_data['resampled'] = true;
//	    if ($limit) $filter_data['limited'] = true;
	}
    } else {
	$mask = NULL;
//	$resample = 0;
//	$limit = 0;
    }

    $log_filter = $this->GetGroupOption($grp, 'filter', array());
    $log_opts = $this->GetGroupOption($grp, 'opts', array());

    $cfg = &$this->groups[$grp->gid];

    if ($cfg['filter']) $log_filter = array_merge($log_filter, $cfg['filter']);
    if ($cfg['opts']) $log_opts = array_merge($log_opts, $cfg['opts']);

    $info = adeiAnalyzeLogs($ivl, $log_filter, $log_opts);
    if ($cfg['logs']) $res = &$info[$cfg['logs']];
    else $res = &$info[$grp->gid];

    return new LOGReaderData($res, $this->GetItemList($grp, $mask));
 }
}

?>